<?php

namespace Sale\Handlers\PaySystem;

use Bitrix\Main,
	Bitrix\Main\Request,
	Bitrix\Main\Localization\Loc,
	Bitrix\Main\Web\HttpClient,
	Bitrix\Sale\Payment,
	Bitrix\Sale\PaySystem,
	Bitrix\Sale\PriceMaths,
	Bitrix\Sale\PaymentCollection,
	Bitrix\Sale\BusinessValue,
	Bitrix\Main\Config\Option;

Loc::loadMessages(__FILE__);

\Bitrix\Main\Loader::includeModule('airbapay.acquiring');
\Bitrix\Main\Loader::includeModule('catalog');
\Bitrix\Main\Loader::includeModule('iblock');

/**
 * Class AirbapayAcquiringHandler
 * @package Sale\Handlers\PaySystem
 */
class AirbapayAcquiringHandler
	extends PaySystem\ServiceHandler implements PaySystem\IRefund
{
	private $helper;

	/**
	 * @param Payment $payment
	 * @param Request|null $request
	 * @return PaySystem\ServiceResult
	 */
	public function initiatePay(Payment $payment, Request $request = null): PaySystem\ServiceResult
	{
		$result = new PaySystem\ServiceResult();

		$requestMain = \Bitrix\Main\Context::getCurrent()->getRequest();
		$protocol = $requestMain->isHttps() ? 'https://' : 'http://';

		$order = $payment->getOrder();
		$basket = $order->getBasket();
		$basketItems = $basket->getBasketItems();

		$helper = $this->getHelper($payment);

		$i = 0;
		do {
			$generatedOrderId = $order->getId() . '_' . $payment->getId() . '_' . rand(1, 999999999);

			$arOrder = \Airbapay\Acquiring\OrderTable::getRow([
				'filter' => [
					'=GENERATED_ORDER_ID' => $generatedOrderId,
				]
			]);
		} while ($i++ < 30 && $arOrder);

		if ($arOrder) {
			$result->addError(new Main\Error('Generate order'));
			return $result;
		}

		$lang = \Bitrix\Main\Application::getInstance()->getContext()->getLanguage();

		$language = 'ru';
		if (in_array($lang, ['ru', 'kz', 'en'])) {
			$language = $lang;
		}

		// $generatedInvoiceId = $order->getId() . '_' . $payment->getId() . '_' . rand(1, 999999999);

		$data = [
			'invoice_id' => $generatedOrderId,
			'amount' => $payment->getSum(),
			'currency' => $payment->getField('CURRENCY'),
			'language' => $language,
			'auto_charge' => $this->getBusinessValue($payment, 'AIRBAPAY_ACQUIRING_AUTO_CHARGE') === 'Y' ? 1 : 0,
			// 'success_callback' => $this->getBusinessValue($payment, 'AIRBAPAY_ACQUIRING_SUCCESS_CALLBACK_URL'),
			// 'failure_callback' => $this->getBusinessValue($payment, 'AIRBAPAY_ACQUIRING_FAILURE_CALLBACK_URL'),
			// 'success_callback' =>  $protocol . SITE_SERVER_NAME . '/bitrix/tools/sale_ps_result.php?airbapay_acquiring_payment_id=' . $payment->getId() . '&paysystem_id=' . $payment->getPaymentSystemId(),
			// 'failure_callback' => $protocol . SITE_SERVER_NAME . '/bitrix/tools/sale_ps_result.php?airbapay_acquiring_payment_id=' . $payment->getId() . '&paysystem_id=' . $payment->getPaymentSystemId(),
			'success_callback' =>  $protocol . SITE_SERVER_NAME . '/bitrix/tools/airbapay_acquiring.php?action=update_status',
			'failure_callback' => $protocol . SITE_SERVER_NAME . '/bitrix/tools/airbapay_acquiring.php?action=update_status',
			'success_back_url' => $this->getBusinessValue($payment, 'AIRBAPAY_ACQUIRING_SUCCESS_BACK_URL'),
			'failure_back_url' => $this->getBusinessValue($payment, 'AIRBAPAY_ACQUIRING_FAILURE_BACK_URL'),
		];

		// echo '<pre>';
		// var_dump($data);
		// echo '</pre>';

		$rsQuery = $helper->apiQuery('payments', $data);
		if ($rsQuery['status'] !== 'success') {
			foreach ($rsQuery['errors'] as $error) {
				$result->addError(new Main\Error($error['message'], $error['code']));
			}

			return $result;
		}

		$rsAdd = \Airbapay\Acquiring\OrderTable::add([
			'GENERATED_ORDER_ID' => $generatedOrderId,
			'PAYMENT_ID' => $payment->getId(),
			'ORDER_ID' => $order->getId(),
			'AIRBAPAY_ID' => $rsQuery['data']['response']['id'],
			'AIRBAPAY_INVOICE_ID' => $rsQuery['data']['response']['invoice_id'],
		]);

		if (!$rsAdd->isSuccess()) {
			$result->addErrors($rsAdd->getErrors());
			return $result;
		}
		

		$this->setExtraParams([
			'redirectUrl' => $rsQuery['data']['response']['redirect_url'],
		]);

		$showTemplateResult = $this->showTemplate($payment, 'template');
		
		if ($showTemplateResult->isSuccess()) {
			$result->setTemplate($showTemplateResult->getTemplate());
		} else {
			$result->addErrors($showTemplateResult->getErrors());
		}

		return $result;
	}

	/**
	 * @param Sale\Payment $payment
	 * @param $refundableSum
	 * @return Sale\PaySystem\ServiceResult
	 * @throws Main\ArgumentException
	 * @throws Main\ArgumentNullException
	 * @throws Main\ArgumentOutOfRangeException
	 * @throws Main\ArgumentTypeException
	 * @throws Main\ObjectException
	 */
	public function refund(\Bitrix\Sale\Payment $payment, $refundableSum): PaySystem\ServiceResult
	{
		$result = new PaySystem\ServiceResult();

		$helper = $this->getHelper($payment);

		$arOrder = \Airbapay\Acquiring\OrderTable::getRow([
			'filter' => [
				'=PAYMENT_ID' => $payment->getId(),
				'=STATUS' => [
					$helper::STATUS_SUCCESS,
					$helper::STATUS_AUTH,
				],
			]
		]);

		if (!$arOrder) {
			$result->addError(new Main\Error('arOrder success not found'));
			return $result;
		}

		// [
		// 	// 'amount' => $refundableSum,
		// 	// 'amount' => $payment->getSum(),
		// ]

		$rsQuery = $helper->apiQuery('payments/return', [
			// 'amount' => $refundableSum,
		], [
			'method' => 'DELETE',
			'payment_id' => $arOrder['AIRBAPAY_ID'],
		]);

		if ($rsQuery['status'] !== 'success') {
			foreach ($rsQuery['errors'] as $error) {
				$result->addError(new Main\Error($error['message'], $error['code']));
			}

			return $result;
		}

		$status = ($rsQuery['data']['response']['status'] ?? false);

		if (!in_array($status, [
			\Airbapay\Acquiring\HelperBitrix::STATUS_RETURN,
			\Airbapay\Acquiring\HelperBitrix::STATUS_UNBLOCK,
		])) {
			$result->addError(new Main\Error('status not return or unblock: ' . $status));

			return $result;
		}

		$result->setOperationType(PaySystem\ServiceResult::MONEY_LEAVING);

		\Airbapay\Acquiring\OrderTable::update($arOrder['ID'], [
			'STATUS' => $status,
		]);

		return $result;
	}

	/**
	 * @param Payment $payment
	 * @return PaySystem\ServiceResult
	 */
	private function createPayment(Payment $payment): PaySystem\ServiceResult
	{
		$result = new PaySystem\ServiceResult();

		return $result;
	}

	private function getAmount($payment)
	{
		return (string)($payment->getSum() * 100);
	}

	private function getCurrencyNumcode($payment)
	{
		$currency = \Bitrix\Currency\CurrencyTable::getList([
			'filter' => ['=CURRENCY' => $payment->getField('CURRENCY')],
			'limit' => 1,
			'select' => ['NUMCODE']
		])->fetch();

		if (!$currency) {
			return false;
		}

		return $currency['NUMCODE'];
	}

	/**
	 * @param Request $request
	 * @return mixed
	 */
	public function getPaymentIdFromRequest(Request $request)
	{
		// $paymentId = $request->getQuery('airbapay_acquiring_payment_id');
		
		// return $paymentId;
	}

	static public function isMyResponse(Request $request, $paySystemId)
    {
		// if ($request->getQuery('airbapay_acquiring_payment_id')) {
		// 	if ($request->getQuery('paysystem_id') == $paySystemId) {
		// 		return true;
		// 	}
		// }

		return false;
    }

	public static function getIndicativeFields()
	{
		// return array('airbapay_acquiring_payment_id');
	}

	/**
	 * @return array
	 */
	public function getCurrencyList(): array
	{
		return ['KZT'];
	}

	/**
	 * @param Payment $payment
	 * @param Request $request
	 * @return PaySystem\ServiceResult
	 */
	public function processRequest(Payment $payment, Request $request): PaySystem\ServiceResult
	{
		$result = new PaySystem\ServiceResult();

		// $data = json_decode(file_get_contents('php://input'), true);

		// if ($data['type'] === 'PAYMENT') {
		// 	$publickey = 'MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAv0pBoPtX5i+FGS16Bqfy'.
		// 	'WI0xNzq/C3t4GDrySZQYOaY7lpaWaZM9sy8mB20PCMKoSkHhYK9Gp3wtmheFJIG7'.
		// 	'd2Shxcj0+VhyvgzhBqdBfx3/CjTEj5OzVtuVhlkVZacrpArrj4GIAk3u4c6Ut7O/'.
		// 	'Hzop/cGzNxKgUXgq31VWP9CTYVFiUe8ArGF7dTuIhN6eVaLBhaViYt32HSKctIt4'.
		// 	'UQCn1azU+8aEkz0qbFXvToXalaIQu8Pzir6epYOlzxWdsmKgtkRH1eGQJRbITJRX'.
		// 	'xK7E+ePi8R9lAYGTlnBJhJNCEp3g7+GgFy91pWqp1tmE5L2VAaHzVXO06mh0Y6J5'.
		// 	'ZwIDAQAB';
			
		// 	preg_replace("(.{64})", "\1\r\n", $publickey);
		// 	$publickey = "-----BEGIN PUBLIC KEY-----\r\n".$publickey."\r\n-----END PUBLIC KEY-----";
	
		// 	$pkey = openssl_pkey_get_public($publickey);
	
		// 	$str = $data['id'] . $data['invoice_id'] . $data['amount'] . $data['currency'] . $data['status'] . $data['description'];
	
		// 	$verify = openssl_verify($str, base64_decode($data['sign']), $pkey, OPENSSL_ALGO_SHA256);

		// 	if ($verify !== 1) {
		// 		$result->addError(new \Bitrix\Main\Error('verify'));
		// 		return $result;
		// 	}

		// 	if ($payment->getSum() !== floatval($data['amount'])) {
		// 		$result->addError(new \Bitrix\Main\Error('amount'));
		// 		return $result;
		// 	}

		// 	$arOrder = \Airbapay\Acquiring\OrderTable::getRow([
		// 		'filter' => [
		// 			'=PAYMENT_ID' => $payment->getId(),
		// 			'=AIRBAPAY_ID' => $data['id'],
		// 			'=AIRBAPAY_INVOICE_ID' => $data['invoice_id'],
		// 		]
		// 	]);

		// 	if (!$arOrder) {
		// 		$result->addError(new \Bitrix\Main\Error('arOrder'));
		// 		return $result;
		// 	}

		// 	if ($data['status'] === \Airbapay\Acquiring\HelperBitrix::STATUS_SUCCESS) {
		// 		$result->setOperationType(PaySystem\ServiceResult::MONEY_COMING);
		// 		$result->setPsData([
		// 			'PS_STATUS' => 'Y'
		// 		]);
		// 	}

		// 	\Airbapay\Acquiring\OrderTable::update($arOrder['ID'], [
		// 		'STATUS' => $data['status'],
		// 	]);
		// }

		return $result;
	}

	/**
	 * @param Payment|null $payment
	 * @return bool
	 */
	protected function isTestMode(Payment $payment = null): bool
	{
		return $this->getBusinessValue($payment, 'AIRBAPAY_ACQUIRING_TEST_MODE') === 'Y';
	}

	/**
	 * @param array $data
	 * @return mixed
	 */
	private static function encode(array $data)
	{
		// return Main\Web\Json::encode($data, JSON_UNESCAPED_UNICODE);
	}

	/**
	 * @param string $data
	 * @return mixed
	 */
	private static function decode($data)
	{
		// try
		// {
		// 	return Main\Web\Json::decode($data);
		// }
		// catch (Main\ArgumentException $exception)
		// {
		// 	return false;
		// }
	}

	private function getHelper(Payment $payment): \Airbapay\Acquiring\Helper
	{
		if ($this->helper === null) {
			$this->helper = new \Airbapay\Acquiring\HelperBitrix([
				'isTestMode' => $this->isTestMode($payment),
				'user' => $this->getBusinessValue($payment, 'AIRBAPAY_ACQUIRING_USER'),
				'password' => $this->getBusinessValue($payment, 'AIRBAPAY_ACQUIRING_PASSWORD'),
				'terminalId' => $this->getBusinessValue($payment, 'AIRBAPAY_ACQUIRING_TERMINAL_ID'),
			]);
		}
		
		return $this->helper;
	}
}
